//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include <IniFiles.hpp>
#include "App_Documents.h"
#include "Moa_AutoPointers.h"
#include "Moa_Clipboard.h"
#include "Moa_DynamicArrays.h"
#include "Moa_Files.h"
#include "Moa_ListViews.h"
#include "Moa_ShellFiles.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_documents {
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentEntry::EnsureValidPath(const AnsiString& APath)
{
	AnsiString LPath = APath;
	try {
		TPathUtilities::AssureAbsolute(LPath);
		LPath = ExpandFileName(LPath);
		LPath = TFileUtilities::GetLongPath(LPath);
	}
	catch(EFileError& LException) {
		throw EDocumentError("The program cannot accept the path.");
	}
	return LPath;
}
//---------------------------------------------------------------------------
TFileTypeCache* __fastcall TDocumentEntry::GetFileTypeCache()
{
	static TFileTypeCache* LFileTypeCache = NULL;
	if(LFileTypeCache == NULL) {
		LFileTypeCache = new TFileTypeCache();
	}
	return LFileTypeCache;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntry::Initialize(const AnsiString& AFile)
{
	FPath = EnsureValidPath(AFile);
	FDirectory = ExtractFileDir(FPath);
	FName = ExtractFileName(FPath);
	FType = GetFileTypeCache()->Determine(FPath);
	WIN32_FIND_DATA LFileData = TFileUtilities::GetData(FPath);
	FSize = TFileDataUtilities::GetSize(LFileData);
	FLastWriteTime = LFileData.ftLastWriteTime;
	FAttributes = LFileData.dwFileAttributes;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntry::Uninitialize()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntry::TDocumentEntry(const AnsiString& AFile) :
	TObject(),
	FPath(),
	FDirectory(),
	FName(),
	FType(),
	FSize(),
	FLastWriteTime(),
	FAttributes(),
	FResult(orNone)
{
	Initialize(AFile);
}
//---------------------------------------------------------------------------
__fastcall TDocumentEntry::~TDocumentEntry()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentEntry::Assign(TDocumentEntry* ASource)
{
	if(ASource == this) {
		return;
	}
	FPath = ASource->FPath;
	FDirectory = ASource->FDirectory;
	FName = ASource->FName;
	FSize = ASource->FSize;
	FType = ASource->FType;
	FLastWriteTime = ASource->FLastWriteTime;
	FResult = ASource->FResult;
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentEntry::IsDirectory()
{
	return (Attributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TDocumentModel::SortCompareByPath(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentDescriber::CompareByPath(
		static_cast<TDocumentModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::SortCompareByDirectory(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentDescriber::CompareByDirectory(
		static_cast<TDocumentModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::SortCompareByName(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentDescriber::CompareByName(
		static_cast<TDocumentModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::SortCompareBySize(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentDescriber::CompareBySize(
		static_cast<TDocumentModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::SortCompareByType(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentDescriber::CompareByType(
		static_cast<TDocumentModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::SortCompareByLastWriteTime(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentDescriber::CompareByLastWriteTime(
		static_cast<TDocumentModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::SortCompareByResult(int AIndex1, int AIndex2, void* AData)
{
	return TDocumentDescriber::CompareByResult(
		static_cast<TDocumentModel*>(AData)->Items[AIndex1],
		static_cast<TDocumentModel*>(AData)->Items[AIndex2]);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::Initialize()
{
	try {
		FList = new TTypedObjectList<TDocumentEntry>();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TDocumentEntry* __fastcall TDocumentModel::GetItem(int AIndex)
{
	return List->Items[AIndex];
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::DoInsertItems(const TIntegerDynArray& AIndexes)
{
	if(OnInsertItems != NULL) {
		OnInsertItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::DoDeleteItems(const TIntegerDynArray& AIndexes)
{
	if(OnDeleteItems != NULL) {
		OnDeleteItems(this, AIndexes);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::DoClear()
{
	if(OnClear != NULL) {
		OnClear(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::DoReorder(const TReorderRecords& ARecords)
{
	if(OnReorder != NULL) {
		OnReorder(this, ARecords);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::DoItemsChange(int AStartIndex, int AEndIndex)
{
	if(OnItemsChange != NULL) {
		OnItemsChange(this, AStartIndex, AEndIndex);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentModel::TDocumentModel() :
	TObject(),
	FList(NULL),
	FOnInsertItems(NULL),
	FOnDeleteItems(NULL),
	FOnClear(NULL),
	FOnReorder(NULL),
	FOnItemsChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocumentModel::~TDocumentModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::AddNewItems(const TStringDynArray& AFiles)
{
	if(AFiles.Length == 0) {
		return;
	}
	TAutoPointer<TTypedObjectList<TDocumentEntry> > LList = new TTypedObjectList<TDocumentEntry>();
	LList->Capacity = AFiles.Length;
	for(int LI = 0; LI < AFiles.Length; ++LI) {
		TAutoPointer<TDocumentEntry> LItem = new TDocumentEntry(AFiles[LI]);
		LList->Add(LItem);
		LItem.Release();
	}
	DynamicArray<TDocumentEntry*> LArray;
	LArray.Length = LList->Count;
	for(int LI = 0; LI < LArray.Length; ++LI) {
		LArray[LI] = LList->Items[LI];
	}
	AddItems(LArray);
	LList->OwnsObjects = false;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::AddItems(const DynamicArray<TDocumentEntry*>& AItems)
{
	if(AItems.Length == 0) {
		return;
	}
	TIntegerDynArray LIndexes;
	LIndexes.Length = AItems.Length;
	TListUtilities::EnsureCapacity(List, List->Count + AItems.Length);
	for(int LI = 0; LI < AItems.Length; ++LI) {
		LIndexes[LI] = List->Add(AItems[LI]);
	}
	DoInsertItems(LIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::DeleteItems(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		List->Delete(AIndexes[AIndexes.High - LI]);
	}
	DoDeleteItems(AIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::Clear()
{
	if(Count == 0) {
		return;
	}
	List->Clear();
	DoClear();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::CutToClipboard(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	CopyToClipboard(AIndexes);
	DeleteItems(AIndexes);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::CopyToClipboard(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	TAutoPointer<TStrings> LLines = new TStringList();
	LLines->Capacity = AIndexes.Length;
	for(int LI = 0; LI < AIndexes.Length; ++LI) {
		LLines->Add(Items[AIndexes[LI]]->Path);
	}
	Clipboard()->AsText = LLines->Text;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::PasteFromClipboard()
{
	TStringDynArray LFiles;
	LFiles = TStringArrayUtilities::FromText(Clipboard()->AsText);
	LFiles = TStringArrayUtilities::RemoveBlankLines(LFiles);
	AddNewItems(LFiles);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::SortByPath()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByPath, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::SortByDirectory()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByDirectory, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::SortByName()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByName, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::SortBySize()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareBySize, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::SortByType()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByType, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::SortByLastWriteTime()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByLastWriteTime, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::SortByResult()
{
	Reorder(TListAlgorithms::Sort(Count, &SortCompareByResult, this));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::ReverseOrder()
{
	Reorder(TListAlgorithms::Reverse(Count));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::MoveUp(const TIntegerDynArray& AIndexes)
{
	Reorder(TListAlgorithms::Slide(Count, AIndexes, -1));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::MoveDown(const TIntegerDynArray& AIndexes)
{
	Reorder(TListAlgorithms::Slide(Count, AIndexes, 1));
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::Reorder(const TReorderRecords& ARecords)
{
	TObjectListUtilities::Reorder(List, ARecords);
	DoReorder(ARecords);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentModel::ItemsChange(int AStartIndex, int AEndIndex)
{
	DoItemsChange(AStartIndex, AEndIndex);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::Find(TDocumentEntry* AItem)
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::FindByResult(TOpenResult AResult)
{
	for(int LI = 0; LI < Count; ++LI) {
		if(TDocumentDescriber::CompareResult(Items[LI]->Result, AResult) == 0) {
			return LI;
		}
	}
	return -1;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TDocumentModel::FindItemsByResult(TOpenResult AResult)
{
	TAutoPointer<TDynamicArrayBuilder<int> > LBuilder = new TDynamicArrayBuilder<int>();
	LBuilder->Capacity = CountByResult(AResult);
	for(int LI = 0; LI < Count; ++LI) {
		if(TDocumentDescriber::CompareResult(Items[LI]->Result, AResult) == 0) {
			LBuilder->Add(LI);
		}
	}
	return LBuilder->Finish();
}
//---------------------------------------------------------------------------
int __fastcall TDocumentModel::CountByResult(TOpenResult AResult)
{
	int LCount = 0;
	for(int LI = 0; LI < Count; ++LI) {
		if(TDocumentDescriber::CompareResult(Items[LI]->Result, AResult) == 0) {
			++LCount;
		}
	}
	return LCount;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TDocumentSelections::GetItem(int AIndex)
{
	return TListViewItemUtilities::IsSelected(ListView, AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::SetItem(int AIndex, bool AItem)
{
	TListViewItemUtilities::SetSelected(ListView, AIndex, AItem);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentSelections::GetCount()
{
	return ListView->Items->Count;
}
//---------------------------------------------------------------------------
int __fastcall TDocumentSelections::GetSelectionCount()
{
	return ListView->SelCount;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TDocumentSelections::GetSelectionIndexes()
{
	return TListViewItemsUtilities::GetSelectionIndexes(ListView);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::SetSelectionIndexes(const TIntegerDynArray& ASelectionIndexes)
{
	TListViewItemsUtilities::SetSelectionIndexes(ListView, ASelectionIndexes);
}
//---------------------------------------------------------------------------
__fastcall TDocumentSelections::TDocumentSelections(TDocumentModel* ADocumentModel, TListView* AListView) :
	FDocumentModel(ADocumentModel),
	FListView(AListView)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TDocumentSelections::~TDocumentSelections()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::InsertItems(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::InsertItems(ListView, AIndexes);
		TListViewItemsUtilities::SetSelectionIndexes(ListView, AIndexes);
		TListViewItemUtilities::SetFocused(ListView, AIndexes[AIndexes.High], true);
		TListViewItemUtilities::EnsureVisible(ListView, AIndexes[AIndexes.High], false);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::DeleteItems(const TIntegerDynArray& AIndexes)
{
	if(AIndexes.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::DeleteItems(ListView, AIndexes);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::Clear()
{
	if(ListView->Items->Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->Items->Count = 0;
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::Reorder(const TReorderRecords& ARecords)
{
	if(ARecords.Length == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		TListViewItemsUtilities::Reorder(ListView, ARecords);
		ListView->Invalidate();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::ItemsChange(int AStartIndex, int AEndIndex)
{
	ListView->Items->BeginUpdate();
	try {
		ListView->UpdateItems(AStartIndex, AEndIndex);
		ListView->Update();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::SelectAll()
{
	if(Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->SelectAll();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::ClearSelection()
{
	if(Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
		ListView->ClearSelection();
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentSelections::ReverseSelection()
{
	if(Count == 0) {
		return;
	}
	ListView->Items->BeginUpdate();
	try {
 		TListViewItemsUtilities::ReverseSelection(ListView);
	}
	catch(...) {
		ListView->Items->EndUpdate();
		throw;
	}
	ListView->Items->EndUpdate();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Initialize()
{
	try {
		FClipboardWatcher = new TClipboardWatcher();
		FClipboardWatcher->Enabled = true;
		FClipboardWatcher->OnChange = ClipboardWatcherChange;
		FAddRegularFilesDialog = new TOpenDialog(NULL);
		FAddRegularFilesDialog->Title = "Add Regular Files";
		FAddRegularFilesDialog->Filter = "All (*.*)|*.*";
		FAddRegularFilesDialog->FilterIndex = 1;
		FAddRegularFilesDialog->Options = FAddRegularFilesDialog->Options
			<< ofAllowMultiSelect
			<< ofFileMustExist;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Uninitialize()
{
	delete FAddRegularFilesDialog;
	FAddRegularFilesDialog = NULL;

	delete FClipboardWatcher;
	FClipboardWatcher = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ClipboardWatcherChange(TObject* ASender)
{
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::TDocumentCommands(TDocumentModel* ADocumentModel, TDocumentSelections* ADocumentSelections) :
	TCommands(),
	FDocumentModel(ADocumentModel),
	FDocumentSelections(ADocumentSelections),
	FClipboardWatcher(NULL),
	FAddRegularFilesDialog(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::~TDocumentCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::New()
{
	if(!CanNew()) {
		HandleUnavailableFeature();
	}
	if(DocumentModel->CountByResult(orSucceeded) != DocumentModel->Count) {
		if(Application->MessageBox(
			"The program clears the document list.",
			Application->Title.c_str(),
			MB_OKCANCEL | MB_ICONWARNING | MB_SETFOREGROUND) != IDOK) {
			Abort();
		}
	}
	DocumentModel->Clear();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanNew()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddRegularFiles()
{
	if(!CanAddRegularFiles()) {
		HandleUnavailableFeature();
	}
	AddRegularFilesDialog->FileName = AnsiString();
	if(!AddRegularFilesDialog->Execute()) {
		Abort();
	}
	DocumentModel->AddNewItems(TStringArrayUtilities::FromList(AddRegularFilesDialog->Files));
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanAddRegularFiles()
{
	return CanAddFiles();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddDirectory()
{
	if(!CanAddDirectory()) {
		HandleUnavailableFeature();
	}
	AnsiString LDirectory = GetCurrentDir();
	if(!SelectDirectory("Specify a directory.", WideString(), LDirectory)) {
		Abort();
	}
	SetCurrentDir(LDirectory);
	DocumentModel->AddNewItems(TDynamicArrayUtilities<AnsiString>::FromElements(LDirectory));
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanAddDirectory()
{
	return CanAddFiles();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::AddFiles(const TStringDynArray& AFiles)
{
	if(!CanAddFiles()) {
		HandleUnavailableFeature();
	}
	DocumentModel->AddNewItems(AFiles);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanAddFiles()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Delete()
{
	if(!CanDelete()) {
		HandleUnavailableFeature();
	}
	if(Application->MessageBox(
		"The program deletes the documents from the list.",
		Application->Title.c_str(),
		MB_OKCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDOK) {
		Abort();
	}
	DocumentModel->DeleteItems(DocumentSelections->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanDelete()
{
	return DocumentSelections->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::RemoveSucceeded()
{
	if(!CanRemoveSucceeded()) {
		HandleUnavailableFeature();
	}
	DocumentModel->DeleteItems(DocumentModel->FindItemsByResult(orSucceeded));
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanRemoveSucceeded()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Cut()
{
	if(!CanCut()) {
		HandleUnavailableFeature();
	}
	DocumentModel->CutToClipboard(DocumentSelections->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanCut()
{
	return CanCopy() && CanDelete();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Copy()
{
	if(!CanCopy()) {
		HandleUnavailableFeature();
	}
	DocumentModel->CopyToClipboard(DocumentSelections->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanCopy()
{
	return DocumentSelections->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Paste()
{
	if(!CanPaste()) {
		HandleUnavailableFeature();
	}
	DocumentModel->PasteFromClipboard();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanPaste()
{
	return Clipboard()->HasFormat(CF_TEXT);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SelectAll()
{
	if(!CanSelectAll()) {
		HandleUnavailableFeature();
	}
	DocumentSelections->SelectAll();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSelectAll()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ReverseSelection()
{
	if(!CanReverseSelection()) {
		HandleUnavailableFeature();
	}
	DocumentSelections->ReverseSelection();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanReverseSelection()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByPath()
{
	if(!CanSortByPath()) {
		HandleUnavailableFeature();
	}
	DocumentModel->SortByPath();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByPath()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByDirectory()
{
	if(!CanSortByDirectory()) {
		HandleUnavailableFeature();
	}
	DocumentModel->SortByDirectory();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByDirectory()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByName()
{
	if(!CanSortByName()) {
		HandleUnavailableFeature();
	}
	DocumentModel->SortByName();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByName()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortBySize()
{
	if(!CanSortBySize()) {
		HandleUnavailableFeature();
	}
	DocumentModel->SortBySize();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortBySize()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByType()
{
	if(!CanSortByType()) {
		HandleUnavailableFeature();
	}
	DocumentModel->SortByType();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByType()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByLastWriteTime()
{
	if(!CanSortByLastWriteTime()) {
		HandleUnavailableFeature();
	}
	DocumentModel->SortByLastWriteTime();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByLastWriteTime()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SortByResult()
{
	if(!CanSortByResult()) {
		HandleUnavailableFeature();
	}
	DocumentModel->SortByResult();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSortByResult()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ReverseOrder()
{
	if(!CanReverseOrder()) {
		HandleUnavailableFeature();
	}
	DocumentModel->ReverseOrder();
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanReverseOrder()
{
	return DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::MoveUp()
{
	if(!CanMoveUp()) {
		HandleUnavailableFeature();
	}
	DocumentModel->MoveUp(DocumentSelections->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanMoveUp()
{
	return DocumentSelections->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::MoveDown()
{
	if(!CanMoveDown()) {
		HandleUnavailableFeature();
	}
	DocumentModel->MoveDown(DocumentSelections->SelectionIndexes);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanMoveDown()
{
	return DocumentSelections->SelectionCount > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::LoadSettings(TMemIniFile* AValues, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".AddRegularFilesDialog";
	{
		AddRegularFilesDialog->FilterIndex = AValues->ReadInteger(
			LSection,
			"FilterIndex",
			AddRegularFilesDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SaveSettings(TMemIniFile* AValues, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".AddRegularFilesDialog";
	{
		AValues->WriteInteger(
			LSection,
			"FilterIndex",
			AddRegularFilesDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::EvaluateResult(TOpenResult AResult)
{
	switch(AResult) {
	case orSucceeded:
		return 0;
	case orFailed:
		return 1;
	case orNone:
		return 2;
	default:
		throw EDocumentError("The program does not support the result.");
	}
}
//---------------------------------------------------------------------------
int TDocumentDescriber::ImageIndex_RegularFile = -1;
int TDocumentDescriber::ImageIndex_Directory = -1;
int TDocumentDescriber::StateIndex_None = -1;
int TDocumentDescriber::StateIndex_Succeeded = -1;
int TDocumentDescriber::StateIndex_Failed = -1;
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribePath(const AnsiString& APath)
{
	return APath;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeType(const AnsiString& AType)
{
	return AType;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeTime(const FILETIME& ATime)
{
	FILETIME LFileTime = {0};
	SYSTEMTIME LSystemTime = {0};
	if((ATime.dwLowDateTime == 0 && ATime.dwHighDateTime == 0) ||
		!::FileTimeToLocalFileTime(&ATime, &LFileTime) ||
		!::FileTimeToSystemTime(&LFileTime, &LSystemTime)) {
		return "(Unknown)";
	}
	return AnsiString().sprintf(
		"%04d-%02d-%02d %02d:%02d:%02d",
		LSystemTime.wYear,
		LSystemTime.wMonth,
		LSystemTime.wDay,
		LSystemTime.wHour,
		LSystemTime.wMinute,
		LSystemTime.wSecond);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeSize(DWORDLONG ASize)
{
	return TAnsiStringUtilities::FormatInteger(ASize, 3, ',');
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeResult(TOpenResult AResult)
{
	switch(AResult) {
	case orNone:
		return "None";
	case orSucceeded:
		return "Succeeded";
	case orFailed:
		return "Failed";
	default:
		return AnsiString("(Unknown)");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeByPath(TDocumentEntry* ADocumentEntry)
{
	return DescribePath(ADocumentEntry->Path);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeByDirectory(TDocumentEntry* ADocumentEntry)
{
	return DescribePath(ADocumentEntry->Directory);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeByName(TDocumentEntry* ADocumentEntry)
{
	return DescribePath(ADocumentEntry->Name);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeBySize(TDocumentEntry* ADocumentEntry)
{
	if(ADocumentEntry->IsDirectory()) {
		return AnsiString();
	}
	return DescribeSize(ADocumentEntry->Size);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeByType(TDocumentEntry* ADocumentEntry)
{
	return DescribeType(ADocumentEntry->Type);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeByLastWriteTime(TDocumentEntry* ADocumentEntry)
{
	return DescribeTime(ADocumentEntry->LastWriteTime);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDocumentDescriber::DescribeByResult(TDocumentEntry* ADocumentEntry)
{
	return DescribeResult(ADocumentEntry->Result);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::ComparePath(const AnsiString& APath1, const AnsiString& APath2)
{
	return AnsiCompareFileName(APath1, APath2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareType(const AnsiString& AType1, const AnsiString& AType2)
{
	return AnsiCompareStr(AType1, AType2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareTime(const FILETIME& ATime1, const FILETIME& ATime2)
{
	return ::CompareFileTime(&ATime1, &ATime2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareSize(DWORDLONG ASize1, DWORDLONG ASize2)
{
	if(ASize1 < ASize2) {
		return -1;
	}
	else if(ASize1 > ASize2) {
		return 1;
	}
	else {
		return 0;
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareResult(TOpenResult AResult1, TOpenResult AResult2)
{
	return EvaluateResult(AResult1) - EvaluateResult(AResult2);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareByPath(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return ComparePath(ADocumentEntry1->Path, ADocumentEntry2->Path);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareByDirectory(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return ComparePath(ADocumentEntry1->Directory, ADocumentEntry2->Directory);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareByName(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return ComparePath(ADocumentEntry1->Name, ADocumentEntry2->Name);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareBySize(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	if(ADocumentEntry1->IsDirectory()) {
		if(ADocumentEntry2->IsDirectory()) {
			return 0;
		}
		else {
			return -1;
		}
	}
	else {
		if(ADocumentEntry2->IsDirectory()) {
			return 1;
		}
		else {
			return CompareSize(ADocumentEntry1->Size, ADocumentEntry2->Size);
		}
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareByType(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return CompareType(ADocumentEntry1->Type, ADocumentEntry2->Type);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareByLastWriteTime(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return CompareTime(ADocumentEntry1->LastWriteTime, ADocumentEntry2->LastWriteTime);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::CompareByResult(TDocumentEntry* ADocumentEntry1, TDocumentEntry* ADocumentEntry2)
{
	return CompareResult(ADocumentEntry1->Result, ADocumentEntry2->Result);
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::DetermineImageIndex(TDocumentEntry* ADocumentEntry)
{
	if(ADocumentEntry->IsDirectory()) {
		return ImageIndex_Directory;
	}
	else {
		return ImageIndex_RegularFile;
	}
}
//---------------------------------------------------------------------------
int __fastcall TDocumentDescriber::DetermineStateIndex(TDocumentEntry* ADocumentEntry)
{
	switch(ADocumentEntry->Result) {
	case orNone:
		return StateIndex_None;
	case orSucceeded:
		return StateIndex_Succeeded;
	case orFailed:
		return StateIndex_Failed;
	default:
		return -1;
	}
}
//---------------------------------------------------------------------------
} // End namespace App_documents
//---------------------------------------------------------------------------

