//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "App_Open.h"
#include "App_ProgressDialog.h"
#include "Moa_AutoPointers.h"
#include "Moa_Tasks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_open {
//---------------------------------------------------------------------------
class EProcessError : public Exception
{
public:
	__fastcall EProcessError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class EProcessCreateError : public EProcessError
{
public:
	__fastcall EProcessCreateError(const AnsiString& AMessage) : EProcessError(AMessage) {}
};
//---------------------------------------------------------------------------
class EProcessUnavailable : public EProcessError
{
public:
	__fastcall EProcessUnavailable(const AnsiString& AMessage) : EProcessError(AMessage) {}
};
//---------------------------------------------------------------------------
class TProcess : public TObject
{
private:
	AnsiString FFile;
	AnsiString FParameters;
	AnsiString FWorkingDirectory;
	int FShowCommand;
	HANDLE FHandle;
	__property AnsiString File = {read=FFile};
	__property AnsiString Parameters = {read=FParameters};
	__property AnsiString WorkingDirectory = {read=FWorkingDirectory};
	__property int ShowCommand = {read=FShowCommand};
protected:
public:
	__fastcall TProcess(const AnsiString& AFile, const AnsiString& AParameters, const AnsiString& AWorkingDirectory, int AShowCommand);
	virtual __fastcall ~TProcess();
	virtual void __fastcall Execute();
	virtual bool __fastcall IsRunning();
	__property HANDLE Handle = {read=FHandle};
};
//---------------------------------------------------------------------------
class TOpenTask : public TTask
{
private:
	TProgramEntry* FProgramEntry;
	TDocumentModel* FDocumentModel;
	int FCurrent;
	int FAll;
	TNotifyEvent FOnNext;
	TProcess* __fastcall PrepareOne();
	TOpenResult __fastcall ProcessOne(TProcess* AProcess);
	void __fastcall UnprepareOne(TOpenResult AResult);
	void __fastcall SynchronizedPrepareOne(void* AData);
	void __fastcall SynchronizedUnprepareOne(void* AData);
	void __fastcall SynchronizedDoNext(void* AData);
protected:
	virtual void __fastcall Execute();
	virtual void __fastcall DoNext();
public:
	__fastcall TOpenTask(TProgramEntry* AProgramEntry, TDocumentModel* ADocumentModel);
	virtual __fastcall ~TOpenTask();
	__property TProgramEntry* ProgramEntry = {read=FProgramEntry};
	__property TDocumentModel* DocumentModel = {read=FDocumentModel};
	__property int Current = {read=FCurrent};
	__property int All = {read=FAll};
	__property TNotifyEvent OnNext = {read=FOnNext, write=FOnNext};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TProcess::TProcess(const AnsiString& AFile, const AnsiString& AParameters, const AnsiString& AWorkingDirectory, int AShowCommand) :
	FFile(AFile),
	FParameters(AParameters),
	FWorkingDirectory(AWorkingDirectory),
	FShowCommand(AShowCommand),
	FHandle(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TProcess::~TProcess()
{
	if(FHandle != NULL && FHandle != INVALID_HANDLE_VALUE) {
		::CloseHandle(FHandle);
		FHandle = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TProcess::Execute()
{
	if(Handle != NULL) {
		throw EProcessError("The program cannot start the process.");
	}
	FHandle == INVALID_HANDLE_VALUE;
	SHELLEXECUTEINFO LExecuteInfo = {0};
	LExecuteInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LExecuteInfo.fMask = SEE_MASK_FLAG_NO_UI | SEE_MASK_NOCLOSEPROCESS;
	LExecuteInfo.lpFile = File.c_str();
	LExecuteInfo.lpParameters = static_cast<LPCSTR>(Parameters.data());
	LExecuteInfo.lpDirectory = static_cast<LPCSTR>(WorkingDirectory.data());
	LExecuteInfo.nShow = ShowCommand;
	if(!::ShellExecuteEx(&LExecuteInfo)) {
		throw EProcessCreateError("The program could not create the process.");
	}
	if(LExecuteInfo.hProcess == NULL) {
		throw EProcessUnavailable("The program could not get the handle of the process.");
	}
	FHandle = LExecuteInfo.hProcess;
}
//---------------------------------------------------------------------------
bool __fastcall TProcess::IsRunning()
{
	if(Handle == NULL || Handle == INVALID_HANDLE_VALUE) {
		return false;
	}
	switch(::WaitForSingleObject(Handle, 0)) {
	case WAIT_ABANDONED:
	case WAIT_OBJECT_0:
		return false;
	default:
		return true;
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TProcess* __fastcall TOpenTask::PrepareOne()
{
	TProcess* LProcess = NULL;
	try {
		Synchronize(&SynchronizedPrepareOne, &LProcess);
	}
	catch(...) {
		delete LProcess;
		throw;
	}
	return LProcess;
}
//---------------------------------------------------------------------------
TOpenResult __fastcall TOpenTask::ProcessOne(TProcess* AProcess)
{
	try {
		DoStatus("Opening a document...");
		AProcess->Execute();
		DoStatus("Waiting for a process to terminate...");
		while(AProcess->IsRunning() && !Canceled) {
			::Sleep(100);
		}
		DoStatus("Succeeded");
	}
	catch(EProcessCreateError& LException) {
		DoStatus("Failed");
		TTaskMessageBox(
			this,
			"The program could not open the document.",
			NULL,
			MB_OK | MB_ICONERROR | MB_SETFOREGROUND).Execute();
		Cancel();
		return orFailed;
	}
	catch(EProcessUnavailable& LException) {
		DoStatus("Succeeded");
		if(TTaskMessageBox(
			this,
			"The program cannot detect the termination of the process.",
			NULL,
			MB_OKCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND).Execute() != IDOK) {
			Cancel();
		}
		return orSucceeded;
	}
	return orSucceeded;
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::UnprepareOne(TOpenResult AResult)
{
	Synchronize(&SynchronizedUnprepareOne, &AResult);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedPrepareOne(void* AData)
{
	// Reset the results
	if(Current == 0) {
		if(DocumentModel->FindByResult(orSucceeded) >= 0) {
			if(Application->MessageBox(
				"The program resets the current results.",
				Application->Title.c_str(),
				MB_OKCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDOK) {
				Abort();
			}
		}
		for(int LI = 0; LI < DocumentModel->Count; ++LI) {
			DocumentModel->Items[LI]->Result = orNone;
		}
		DocumentModel->ItemsChange(0, DocumentModel->Count - 1);
	}

	// Prepare a process
	DoNext();
	TAutoPointer<TProcess> LProcess;
	if(ProgramEntry->Default) {
		LProcess = new TProcess(
			DocumentModel->Items[Current]->Path,
			AnsiString(),
			DocumentModel->Items[Current]->Directory,
			SW_SHOWNORMAL);
	}
	else {
		LProcess = new TProcess(
			ProgramEntry->Parameters->Target,
			AnsiQuotedStr(DocumentModel->Items[Current]->Path, '\"'),
			ProgramEntry->Parameters->WorkingDirectory,
			ProgramEntry->Parameters->ShowCommand);
	}
	*static_cast<TProcess**>(AData) = LProcess.Release();
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedUnprepareOne(void* AData)
{
	// Set the result
	DocumentModel->Items[Current]->Result = *static_cast<TOpenResult*>(AData);

	// Report the progress
	++FCurrent;
	ReportProgress(Current, All);
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::SynchronizedDoNext(void* AData)
{
	if(OnNext != NULL) {
		OnNext(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::Execute()
{
	while(Current < All) {
		TAutoPointer<TProcess> LProcess;
		UnprepareOne(ProcessOne(LProcess = PrepareOne()));
		Continue();
	}
}
//---------------------------------------------------------------------------
void __fastcall TOpenTask::DoNext()
{
	Synchronize(&SynchronizedDoNext, NULL);
}
//---------------------------------------------------------------------------
__fastcall TOpenTask::TOpenTask(TProgramEntry* AProgramEntry, TDocumentModel* ADocumentModel) :
	TTask(),
	FProgramEntry(AProgramEntry),
	FDocumentModel(ADocumentModel),
	FCurrent(0),
	FAll(DocumentModel->Count),
	FOnNext(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TOpenTask::~TOpenTask()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TOpenModel::TaskProgress(TObject* ASender, int ACurrent, int AAll)
{
	Task->DocumentModel->ItemsChange(ACurrent, ACurrent);
	AppProgressDialog->Values->Program = AnsiString();
	AppProgressDialog->Values->Document = AnsiString();
	AppProgressDialog->Values->Status = AnsiString();
	AppProgressDialog->Values->Current = Task->Current;
	AppProgressDialog->Values->All = Task->All;
}
//---------------------------------------------------------------------------
void __fastcall TOpenModel::TaskStatus(TObject* ASender, const AnsiString& AStatus)
{
	AppProgressDialog->Values->Status = AStatus;
}
//---------------------------------------------------------------------------
void __fastcall TOpenModel::TaskNext(TObject* ASender)
{
	AppProgressDialog->Values->Program = Task->ProgramEntry->Name;
	AppProgressDialog->Values->Document = Task->DocumentModel->Items[Task->Current]->Name;
	AppProgressDialog->Values->Status = AnsiString();
}
//---------------------------------------------------------------------------
void __fastcall TOpenModel::TaskTerminate(TObject* ASender)
{
	AppProgressDialog->Finish();
	AppProgressDialog->Values->Reset();
}
//---------------------------------------------------------------------------
void __fastcall TOpenModel::AppProgressDialogExecute(TObject* ASender)
{
	Task->Perform();
}
//---------------------------------------------------------------------------
void __fastcall TOpenModel::AppProgressDialogCancel(TObject* ASender)
{
	Task->Cancel();
}
//---------------------------------------------------------------------------
__fastcall TOpenModel::TOpenModel() :
	TObject(),
	FTask(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TOpenModel::~TOpenModel()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TOpenModel::Perform(TProgramEntry* AProgramEntry, TDocumentModel* ADocumentModel)
{
	try {
		TAutoPointer<TOpenTask> LTask = new TOpenTask(AProgramEntry, ADocumentModel);
		LTask->OnTerminate = TaskTerminate;
		LTask->OnProgress = TaskProgress;
		LTask->OnStatus = TaskStatus;
		LTask->OnNext = TaskNext;
		FTask = LTask;
		TAppProgressDialog::EnsureCreated();
		AppProgressDialog->Values->Reset();
		AppProgressDialog->OnExecute = AppProgressDialogExecute;
		AppProgressDialog->OnCancel = AppProgressDialogCancel;
		AppProgressDialog->Execute();
	}
	__finally {
		FTask = NULL;
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TOpenCommands::TOpenCommands(TProgramModel* AProgramModel, TProgramSelections* AProgramSelections, TDocumentModel* ADocumentModel, TDocumentSelections* ADocumentSelections, TOpenModel* AOpenModel) :
	TCommands(),
	FProgramModel(AProgramModel),
	FProgramSelections(AProgramSelections),
	FDocumentModel(ADocumentModel),
	FDocumentSelections(ADocumentSelections),
	FOpenModel(AOpenModel)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TOpenCommands::~TOpenCommands()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TOpenCommands::Perform()
{
	if(!CanPerform()) {
		HandleUnavailableFeature();
	}
	OpenModel->Perform(
		ProgramModel->Items[ProgramSelections->SelectionIndex],
		DocumentModel);
}
//---------------------------------------------------------------------------
bool __fastcall TOpenCommands::CanPerform()
{
	return 
		ProgramSelections->SelectionIndex >= 0 &&
		DocumentModel->Count > 0;
}
//---------------------------------------------------------------------------
} // End namespace App_open
//---------------------------------------------------------------------------

