//---------------------------------------------------------------------------
#ifndef App_ProgramsH
#define App_ProgramsH
//---------------------------------------------------------------------------
#include <StdCtrls.hpp>
#include "App_Commands.h"
#include "Moa_TypedLists.h"
//---------------------------------------------------------------------------
namespace App_programs {
//---------------------------------------------------------------------------
class EProgramError : public Exception
{
public:
	__fastcall EProgramError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TShortcutParameters : public TObject
{
private:
	AnsiString FTarget;
	AnsiString FWorkingDirectory;
	int FShowCommand;
	void __fastcall Initialize();
	void __fastcall Initialize(const AnsiString& AFile);
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TShortcutParameters();
	__fastcall TShortcutParameters(const AnsiString& AFile);
	virtual __fastcall ~TShortcutParameters();
	virtual void __fastcall Assign(TShortcutParameters* AOther);
	__property AnsiString Target = {read=FTarget};
	__property AnsiString WorkingDirectory = {read=FWorkingDirectory};
	__property int ShowCommand = {read=FShowCommand};
};
//---------------------------------------------------------------------------
class TProgramEntry : public TObject
{
private:
	TIcon* FIcon;
	AnsiString FFile;
	AnsiString FName;
	TShortcutParameters* FParameters;
	bool FDefault;
	static AnsiString __fastcall EnsureValidPath(const AnsiString& APath);
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TProgramEntry();
	__fastcall TProgramEntry(const AnsiString& AFile);
	virtual __fastcall ~TProgramEntry();
	virtual void __fastcall Start();
	virtual void __fastcall Explore();
	virtual void __fastcall ShowProperties();
	virtual void __fastcall Assign(TProgramEntry* AOther);
	__property TIcon* Icon = {read=FIcon};
	__property AnsiString File = {read=FFile};
	__property AnsiString Name = {read=FName};
	__property TShortcutParameters* Parameters = {read=FParameters};
	__property bool Default = {read=FDefault};
};
//---------------------------------------------------------------------------
class TProgramModel : public TObject
{
private:
	static const AnsiString ProgramsDirectory;
	static const AnsiString ReadmeFile;
	TTypedObjectList<TProgramEntry>* FList;
	TNotifyEvent FOnReload;
	static void __fastcall EnsureProgramsDirectoryExists();
	static void __fastcall EnsureReadmeFileExists();
	static int __fastcall ReloadCompare(void* AItem1, void* AItem2);
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TProgramEntry* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	__property TTypedObjectList<TProgramEntry>* List = {read=FList};
protected:
	virtual void __fastcall DoReload();
public:
	__fastcall TProgramModel();
	virtual __fastcall ~TProgramModel();
	virtual void __fastcall Organize();
	virtual void __fastcall Reload();
	__property TProgramEntry* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property TNotifyEvent OnReload = {read=FOnReload, write=FOnReload};
};
//---------------------------------------------------------------------------
class TProgramSelections : public TObject
{
private:
	TProgramModel* FProgramModel;
	TComboBox* FComboBox;
	bool __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	int __fastcall GetSelectionCount();
	int __fastcall GetSelectionIndex();
	void __fastcall SetSelectionIndex(int ASelectionIndex);
	__property TProgramModel* ProgramModel = {read=FProgramModel};
	__property TComboBox* ComboBox = {read=FComboBox};
protected:
public:
	__fastcall TProgramSelections(TProgramModel* AProgramModel, TComboBox* AComboBox);
	virtual __fastcall ~TProgramSelections();
	virtual void __fastcall Reload();
	__property bool Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
	__property int SelectionCount = {read=GetSelectionCount};
	__property int SelectionIndex = {read=GetSelectionIndex, write=SetSelectionIndex};
};
//---------------------------------------------------------------------------
class TProgramCommands : public TCommands
{
private:
	TProgramModel* FProgramModel;
	TProgramSelections* FProgramSelections;
	__property TProgramModel* ProgramModel = {read=FProgramModel};
	__property TProgramSelections* ProgramSelections = {read=FProgramSelections};
protected:
public:
	__fastcall TProgramCommands(TProgramModel* AProgramModel, TProgramSelections* AProgramSelections);
	virtual __fastcall ~TProgramCommands();
	virtual void __fastcall Organize();
	virtual bool __fastcall CanOrganize();
	virtual void __fastcall Reload();
	virtual bool __fastcall CanReload();
	virtual void __fastcall Start();
	virtual bool __fastcall CanStart();
	virtual void __fastcall Explore();
	virtual bool __fastcall CanExplore();
	virtual void __fastcall ShowProperties();
	virtual bool __fastcall CanShowProperties();
	virtual void __fastcall LoadSettings(TMemIniFile* AValues, const AnsiString& ASection);
	virtual void __fastcall SaveSettings(TMemIniFile* AValues, const AnsiString& ASection);
};
//---------------------------------------------------------------------------
} // End namespace App_programs
using namespace App_programs;
//---------------------------------------------------------------------------
#endif

