//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_Settings.h"
#include "Moa_AutoPointers.h"
#include "Moa_Strings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_settings {
//---------------------------------------------------------------------------
const AnsiString TSettingsModel::SettingsFile = ExtractFilePath(Application->ExeName) + "Settings.ini";
//---------------------------------------------------------------------------
void __fastcall TSettingsModel::Initialize()
{
	try {
		FSections = new TStringList();
		static_cast<TStringList*>(FSections)->CaseSensitive = false;
		static_cast<TStringList*>(FSections)->Duplicates = dupError;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsModel::Uninitialize()
{
	delete FSections;
	FSections = NULL;
}
//---------------------------------------------------------------------------
__fastcall TSettingsModel::TSettingsModel() :
	TObject(),
	FSections(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TSettingsModel::~TSettingsModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TSettingsModel::Load()
{
	TAutoPointer<TMemIniFile> LValues = new TMemIniFile(SettingsFile);
	for(int LI = 0; LI < Sections->Count; ++LI) {
		static_cast<TCommands*>(Sections->Objects[LI])->LoadSettings(
			LValues,
			Sections->Strings[LI]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsModel::Save()
{
	TAutoPointer<TMemIniFile> LValues = new TMemIniFile(SettingsFile);
	LValues->Clear();
	for(int LI = 0; LI < Sections->Count; ++LI) {
		static_cast<TCommands*>(Sections->Objects[LI])->SaveSettings(
			LValues,
			Sections->Strings[LI]);
	}
	LValues->UpdateFile();
}
//---------------------------------------------------------------------------
void __fastcall TSettingsModel::Register(const AnsiString& ASection, TCommands* ACommands)
{
	try {
		Sections->AddObject(ASection, ACommands);
	}
	catch(EListError& LException) {
		throw ESettingsError("The program could not register the section.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsModel::Unregister(const AnsiString& ASection)
{
	try {
		Sections->Delete(Sections->IndexOf(ASection));
	}
	catch(EListError& LException) {
		throw ESettingsError("The program could not unregister the section.");
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TSettingsCommands::TSettingsCommands(TSettingsModel* ASettingsModel) :
	TCommands(),
	FSettingsModel(ASettingsModel)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSettingsCommands::~TSettingsCommands()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSettingsCommands::Load()
{
	if(!CanLoad()) {
		HandleUnavailableFeature();
	}
	SettingsModel->Load();
}
//---------------------------------------------------------------------------
bool __fastcall TSettingsCommands::CanLoad()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TSettingsCommands::Save()
{
	if(!CanSave()) {
		HandleUnavailableFeature();
	}
	SettingsModel->Save();
}
//---------------------------------------------------------------------------
bool __fastcall TSettingsCommands::CanSave()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TSettingsCommands::LoadSettings(TMemIniFile* AValues, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".System";
	{
		SetCurrentDir(TAnsiStringUtilities::Dequote(AValues->ReadString(
			LSection,
			"CurrentDirectory",
			TAnsiStringUtilities::Quote(GetCurrentDir()))));
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsCommands::SaveSettings(TMemIniFile* AValues, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".System";
	{
		AValues->WriteString(
			LSection,
			"CurrentDirectory",
			TAnsiStringUtilities::Quote(GetCurrentDir()));
	}
}
//---------------------------------------------------------------------------
} // End namespace App_settings
//---------------------------------------------------------------------------

