//---------------------------------------------------------------------------
#ifndef Moa_ListsH
#define Moa_ListsH
//---------------------------------------------------------------------------
// Version: 2.0.0
// Date: 2012-04-02 UTC+09:00
// Version: 1.0.0
// Date: 2011-08-26 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_lists {
//---------------------------------------------------------------------------
struct TExchangeRecord
{
	int Index1;
	int Index2;
	__fastcall TExchangeRecord();
	__fastcall TExchangeRecord(int AIndex1, int AIndex2);
	__fastcall TExchangeRecord(const TExchangeRecord& ASource);
	TExchangeRecord& __fastcall operator =(const TExchangeRecord& ASource);
	bool __fastcall operator ==(const TExchangeRecord& AOther) const;
	bool __fastcall operator !=(const TExchangeRecord& AOther) const;
	TExchangeRecord& __fastcall Reverse();
};
typedef DynamicArray<TExchangeRecord> TExchangeRecords;
//---------------------------------------------------------------------------
struct TReorderRecord
{
	int SourceIndex;
	int DestinationIndex;
	__fastcall TReorderRecord();
	__fastcall TReorderRecord(int ASourceIndex, int ADestinationIndex);
	__fastcall TReorderRecord(const TReorderRecord& ASource);
	TReorderRecord& __fastcall operator =(const TReorderRecord& ASource);
	bool __fastcall operator ==(const TReorderRecord& AOther) const;
	bool __fastcall operator !=(const TReorderRecord& AOther) const;
	TReorderRecord& __fastcall Reverse();
};
typedef DynamicArray<TReorderRecord> TReorderRecords;
//---------------------------------------------------------------------------
typedef int __fastcall (* TSortCompare)(int AIndex1, int AIndex2, void* AData);
//---------------------------------------------------------------------------
class TListAlgorithms
{
private:
	struct TSortRecord
	{
		int Index;
		TSortCompare Compare;
		void* Data;
	};
	struct TSlideRecord
	{
		int Index;
		bool Target;
	};
	static int __fastcall SortCompare(void* AItem1, void* AItem2);
	static void __fastcall Swap(int& AValue1, int& AValue2);
protected:
public:
	static TReorderRecords __fastcall Sort(int ACount, TSortCompare ACompare, void* AData);
	static TReorderRecords __fastcall Reverse(int ACount);
	static TReorderRecords __fastcall Slide(int ACount, const TIntegerDynArray& AIndexes, int AStep);
	static TExchangeRecords __fastcall ReverseExchangeRecords(const TExchangeRecords& ARecords);
	static TReorderRecords __fastcall ExchangeRecordsToReorderRecords(const TExchangeRecords& ARecords);
	static TReorderRecords __fastcall ReverseReorderRecords(const TReorderRecords& ARecords);
	static TExchangeRecords __fastcall ReorderRecordsToExchangeRecords(const TReorderRecords& ARecords);
};
//---------------------------------------------------------------------------
class TListUtilities
{
private:
protected:
public:
	static void __fastcall InsertItems(TList* AList, const TIntegerDynArray& AIndexes);
	static void __fastcall InsertSucceedingItems(TList* AList, int AIndex, int ACount);
	static void __fastcall DeleteItems(TList* AList, const TIntegerDynArray& AIndexes);
	static void __fastcall DeleteSucceedingItems(TList* AList, int AIndex, int ACount);
	static void __fastcall ExchangeItems(TList* AList, const TExchangeRecords& ARecords);
	static void __fastcall Reorder(TList* AList, const TReorderRecords& ARecords);
	static void __fastcall Reverse(TList* AList);
	static void* __fastcall Replace(TList* AList, int AIndex, void* AItem);
	static void __fastcall EnsureCapacity(TList* AList, int ACapacity);
};
//---------------------------------------------------------------------------
class TObjectListUtilities : public TListUtilities
{
private:
protected:
public:
	using TListUtilities::Reorder;
	static void __fastcall Reorder(TObjectList* AList, const TReorderRecords& ARecords);
	using TListUtilities::Replace;
	static TObject* __fastcall Replace(TObjectList* AList, int AIndex, TObject* AItem);
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TInsertItemsEvent)(TObject* ASender, const TIntegerDynArray& AIndexes);
typedef void __fastcall (__closure * TDeleteItemsEvent)(TObject* ASender, const TIntegerDynArray& AIndexes);
typedef void __fastcall (__closure * TExchangeItemsEvent)(TObject* ASender, const TExchangeRecords& ARecords);
typedef void __fastcall (__closure * TReorderEvent)(TObject* ASender, const TReorderRecords& ARecords);
typedef void __fastcall (__closure * TItemsChangeEvent)(TObject* ASender, int AStartIndex, int AEndIndex);
typedef void __fastcall (__closure * TUpdateItemsEvent)(TObject* ASender, int AStartIndex, int AEndIndex);
//---------------------------------------------------------------------------
} // End namespace Moa_lists
using namespace Moa_lists;
//---------------------------------------------------------------------------
#endif

