//---------------------------------------------------------------------------
#ifndef Moa_ShellFilesH
#define Moa_ShellFilesH
//---------------------------------------------------------------------------
// Version: 1.0.1
// Date: 2012-05-07 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_shellfiles {
//---------------------------------------------------------------------------
class EShellFileError : public Exception
{
public:
	__fastcall EShellFileError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
enum TIconScale
{
	isShellSized,
	isSmall,
	isLarge,
};
//---------------------------------------------------------------------------
enum TIconEffect
{
	ieLinkOverlay,
	ieSelected,
};
typedef Set<TIconEffect, ieLinkOverlay, ieSelected> TIconEffects;
//---------------------------------------------------------------------------
class TShellFileUtilities
{
private:
	static UINT __fastcall IconScaleToFlags(TIconScale AIconScale);
	static UINT __fastcall IconEffectsToFlags(const TIconEffects& AIconEffects);
	static AnsiString __fastcall EnsureValidExtension(const AnsiString& AExtension);
protected:
public:
	static const TIconScale DefaultIconScale;
	static HIMAGELIST __fastcall GetImageList();
	static HIMAGELIST __fastcall GetImageList(TIconScale AIconScale);
	static void __fastcall GetIcon(int AIndex, TIcon* AIcon);
	static void __fastcall GetIcon(int AIndex, TIcon* AIcon, TIconScale AIconScale);
	static void __fastcall GetIcon(int AIndex, TIcon* AIcon, TIconScale AIconScale, UINT ADrawStyle);
	static void __fastcall GetIconByPath(const AnsiString& APath, TIcon* AIcon);
	static void __fastcall GetIconByPath(const AnsiString& APath, TIcon* AIcon, TIconScale AIconScale);
	static void __fastcall GetIconByPath(const AnsiString& APath, TIcon* AIcon, TIconScale AIconScale, const TIconEffects& AIconEffects);
	static void __fastcall GetIconByExtension(const AnsiString& AExtension, TIcon* AIcon);
	static void __fastcall GetIconByExtension(const AnsiString& AExtension, TIcon* AIcon, TIconScale AIconScale);
	static void __fastcall GetIconByExtension(const AnsiString& AExtension, TIcon* AIcon, TIconScale AIconScale, const TIconEffects& AIconEffects);
	static void __fastcall GetIconForDirectory(TIcon* AIcon);
	static void __fastcall GetIconForDirectory(TIcon* AIcon, TIconScale AIconScale);
	static void __fastcall GetIconForDirectory(TIcon* AIcon, TIconScale AIconScale, const TIconEffects& AIconEffects);
	static int __fastcall FindIconByPath(const AnsiString& APath);
	static int __fastcall FindIconByPath(const AnsiString& APath, TIconScale AIconScale);
	static int __fastcall FindIconByExtension(const AnsiString& AExtension);
	static int __fastcall FindIconByExtension(const AnsiString& AExtension, TIconScale AIconScale);
	static int __fastcall FindIconForDirectory();
	static int __fastcall FindIconForDirectory(TIconScale AIconScale);
};
//---------------------------------------------------------------------------
class TFileTypeCache : public TObject
{
private:
	class TEntry : public TObject
	{
	private:
		AnsiString FDescription;
		static AnsiString __fastcall Determine(const AnsiString& AFile, bool AAsDirectory);
	protected:
	public:
		__fastcall TEntry();
		__fastcall TEntry(const AnsiString& AFile);
		virtual __fastcall ~TEntry();
		__property AnsiString Description = {read=FDescription};
	};
	TStringList* FExtensions;
	TEntry* FDirectory;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	__property TStringList* Extensions = {read=FExtensions};
	__property TEntry* Directory = {read=FDirectory};
protected:
public:
	__fastcall TFileTypeCache();
	virtual __fastcall ~TFileTypeCache();
	virtual AnsiString __fastcall Determine(const AnsiString& AFile);
	virtual AnsiString __fastcall DetermineAsRegularFile(const AnsiString& AFile);
	virtual AnsiString __fastcall DetermineAsDirectory();
	virtual void __fastcall Clear();
};
//---------------------------------------------------------------------------
} // namespace Moa_shellfiles
using namespace Moa_shellfiles;
//---------------------------------------------------------------------------
#endif

