//---------------------------------------------------------------------------
#ifndef Moa_TasksH
#define Moa_TasksH
//---------------------------------------------------------------------------
// Version: 1.0.1
// Date: 2012-04-13 UTC+09:00
// Version: 1.0.0
// Date: 2012-03-31 UTC+09:00
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_tasks {
//---------------------------------------------------------------------------
class ETaskError : public Exception
{
public:
	__fastcall ETaskError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class ETaskCancel : public EAbort
{
public:
	__fastcall ETaskCancel(const AnsiString& AMessage) : EAbort(AMessage) {}
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TTaskSynchronizeMethod)(void* AData);
typedef void __fastcall (__closure * TTaskProgressEvent)(TObject* ASender, int ACurrent, int AAll);
typedef void __fastcall (__closure * TTaskStatusEvent)(TObject* ASender, const AnsiString& AStatus);
typedef void __fastcall (__closure * TTaskExceptionEvent)(TObject* ASender, Exception* AException);
//---------------------------------------------------------------------------
class TTaskThread;
class TTask : public TObject
{
private:
	TTaskThread* FThread;
	bool FStarted;
	bool FCanceled;
	bool FFreeOnTerminate;
	TTaskSynchronizeMethod FSynchronizeMethod;
	void* FSynchronizeData;
	TNotifyEvent FOnExecute;
	TNotifyEvent FOnTerminate;
	TTaskProgressEvent FOnProgress;
	TTaskStatusEvent FOnStatus;
	TTaskExceptionEvent FOnException;
	void __fastcall SynchronizedDoExecute(void* AData);
	void __fastcall SynchronizedDoTerminate(void* AData);
	void __fastcall SynchronizedDoProgress(void* AData);
	void __fastcall SynchronizedDoStatus(void* AData);
	void __fastcall SynchronizedDoException(void* AData);
	void __fastcall SynchronizedSynchronize();
	void __fastcall ThreadExecute(TObject* ASender);
	void __fastcall ThreadTerminate(TObject* ASender);
	__property TTaskThread* Thread = {read=FThread};
	__property bool Started = {read=FStarted};
	__property TTaskSynchronizeMethod SynchronizeMethod = {read=FSynchronizeMethod};
	__property void* SynchronizeData = {read=FSynchronizeData};
protected:
	virtual void __fastcall Execute();
	virtual void __fastcall DoExecute();
	virtual void __fastcall DoTerminate();
	virtual void __fastcall DoProgress(int ACurrent, int AAll);
	virtual void __fastcall DoStatus(const AnsiString& AStatus);
	virtual void __fastcall DoException(Exception* AException);
public:
	__fastcall TTask();
	virtual __fastcall ~TTask();
	virtual void __fastcall Perform();
	virtual void __fastcall Cancel();
	virtual void __fastcall Continue();
	virtual void __fastcall ReportProgress(int ACurrent, int AAll);
	virtual void __fastcall ReportStatus(const AnsiString& AStatus);
	virtual void __fastcall ReportException(Exception* AException);
	virtual void __fastcall Synchronize(TTaskSynchronizeMethod AMethod, void* AData);
	__property bool Canceled = {read=FCanceled};
	__property bool FreeOnTerminate = {read=FFreeOnTerminate, write=FFreeOnTerminate};
	__property TNotifyEvent OnExecute = {read=FOnExecute, write=FOnExecute};
	__property TNotifyEvent OnTerminate = {read=FOnTerminate, write=FOnTerminate};
	__property TTaskProgressEvent OnProgress = {read=FOnProgress, write=FOnProgress};
	__property TTaskStatusEvent OnStatus = {read=FOnStatus, write=FOnStatus};
	__property TTaskExceptionEvent OnException = {read=FOnException, write=FOnException};
};
//---------------------------------------------------------------------------
class TTaskMessageBox
{
private:
	TTask* FTask;
	const char* FMessage;
	const char* FTitle;
	int FFlags;
	int FResult;
	void __fastcall SynchronizedExecute(void* AData);
protected:
public:
	__fastcall TTaskMessageBox(TTask* ATask, const char* AMessage, const char* ATitle, int AFlags);
	virtual __fastcall ~TTaskMessageBox();
	virtual int __fastcall Execute();
};
//---------------------------------------------------------------------------
} // End namespace Moa_tasks
using namespace Moa_tasks;
//---------------------------------------------------------------------------
#endif

